;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Urchins (sprites 3A & 3B), by imamelia
;;
;; This is a disassembly of sprites 3A and 3B in SMW, the non-wall-following Urchins.
;;
;; Uses first extra bit: YES
;;
;; If the extra bit is clear, this sprite will act like sprite 3A, which moves a fixed
;; horizontal/vertical distance.  If the extra bit is set, this sprite will act like sprite
;; 3B, which moves until it hits a surface.  Either way, it will move horizontally or
;; vertically depending on its X position.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

XSpeed:
db $08,$00

YSpeed:
db $00,$08

XDisp:
db $08,$00,$10,$00,$10

YDisp:
db $08,$00,$00,$10,$10

TileProp:
db $37,$37,$77,$B7,$F7

Tilemap:
db $C4,$C6,$C8,$C6

Frame:
db $00,$01,$02,$01

!EyesTile1 = $CA
!EyesTile2 = $CC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
PHB
PHK
PLB
JSR UrchinInit
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR UrchinMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UrchinInit:

LDA $E4,x	;
LDY #$00		;
AND #$10	;
STA $151C,x	;
BNE InitHorizontal	;
INY		;
InitHorizontal:	;
LDA XSpeed,y	;
STA $B6,x		;
LDA YSpeed,y	;
STA $AA,x	;
INC $164A,x	;
RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UrchinMain:

JSL $018032	; interact with sprites
JSL $01ACF9	; get a random number
ORA $9D		;
BNE NoSet1	; if sprites are locked or the number was not 0...
LDA #$0C		; don't set this timer
STA $1558,x	;
NoSet1:		;

JSR UrchinGFX	;

LDA $14C8,x	;
CMP #$08		; if the sprite status is normal...
BEQ ContinueMain	; continue
STZ $1528,x	;
LDA #$FF		;
STA $1558,x	;
RTS		;

ContinueMain:	;

LDA $9D		; return if sprites are locked
BNE Return00	;

JSR SubOffscreenX3	;
JSL $01A7DC	;

; removed sprite number checks ($2E, $3C, $A5, $A6)

LDA $C2,x	;
AND #$01	; only 2 pointers here
JSL $0086DF	; jump to a pointer depending on bit 0 of the sprite state

dw Stationary	;
dw Moving	;

Stationary:	;

LDA $1540,x	; if the state timer is set...
BNE Return00	; return
LDA #$80		; if it is not set...
STA $1540,x	; set it
INC $C2,x	; and increment the sprite state

Return00:		;
RTS		;

Moving:		;

LDA $7FAB10,x	;
AND #$04	; if the extra bit is set and the sprite acts like 3B...
BNE NoTurn	; don't turn when the sprite state timer runs out

LDA $1540,x	; if the extra bit is clear and the sprite acts like 3A...
BEQ ReverseSpeed	; change direction with the sprite state timer runs out

NoTurn:		;

JSL $018022	; update sprite X position
JSL $01801A	; update sprite Y position
JSL $019138	; interact with objects

LDA $1588,x	;
AND #$0F	; if the sprite is touching a wall...
BEQ Return00	;

ReverseSpeed:	;

LDA $B6,x	;
EOR #$FF		; flip the X speed
INC		;
STA $B6,x		;

LDA $AA,x	;
EOR #$FF		; flip the Y speed
INC		;
STA $AA,x	;

LDA #$40		;
STA $1540,x	; set the time until the sprite state changes
INC $C2,x	; increment the sprite state

RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UrchinGFX:

LDA $163E,x	;
BNE SkipReset1	;
INC $1528,x	; not sure what the purpose of these 5 lines is...
LDA #$0C		;
STA $163E,x	; change the frame shown every 0C frames?
SkipReset1:	;

LDA $1528,x	;
AND #$03	;
TAY		;
LDA Frame,y	;
STA $1602,x	;

JSR GetDrawInfo	;

STZ $05		;
LDA $1602,x	;
STA $02		;
LDA $1558,x	;
STA $03		;

GFXLoop:		;

LDX $05		; tilemap index

LDA $00		;
CLC		;
ADC XDisp,x	;
STA $0300,y	;

LDA $01		;
CLC		;
ADC YDisp,x	;
STA $0301,y	;

LDA TileProp,x	;
STA $0303,y	;

CPX #$00		; if the tilemap index is 00...
BNE NotEyesYet	;
LDA #!EyesTile1	; then draw the eyes
LDX $03		;
BEQ StoreTile	;
LDA #!EyesTile2	;
BRA StoreTile	;

NotEyesYet:	;

LDX $02		;
LDA Tilemap,x	;
StoreTile:		;
STA $0302,y	;

INY #4		;
INC $05		;
LDA $05		;
CMP #$05		; if we've drawn 5 tiles...
BNE GFXLoop	; end the loop

LDX $15E9	;
LDY #$02		;
LDA #$04		;
JSL $81B7B3	;

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

;SubOffscreenX0:
;LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
SubOffscreenX3:
LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS











